@@ -43,12 +43,14 @@ AFSFlushBuffers( IN PDEVICE_OBJECT LibDeviceObject,
                  IN PIRP Irp)
 {
 
+    UNREFERENCED_PARAMETER(LibDeviceObject);
     NTSTATUS           ntStatus = STATUS_SUCCESS;
     IO_STACK_LOCATION *pIrpSp = IoGetCurrentIrpStackLocation( Irp);
     PFILE_OBJECT       pFileObject = pIrpSp->FileObject;
     AFSFcb            *pFcb = (AFSFcb *)pFileObject->FsContext;
     AFSCcb            *pCcb = (AFSCcb *)pFileObject->FsContext2;
     IO_STATUS_BLOCK    iosb = {0};
+    BOOLEAN            bReleaseSectionObject = FALSE;
 
     pIrpSp = IoGetCurrentIrpStackLocation( Irp);
 
@@ -58,10 +60,10 @@ AFSFlushBuffers( IN PDEVICE_OBJECT LibDeviceObject,
         if( pFcb == NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSFlushBuffers Attempted access (%08lX) when pFcb == NULL\n",
-                          Irp);
+                          "AFSFlushBuffers Attempted access (%p) when pFcb == NULL\n",
+                          Irp));
 
             try_return( ntStatus = STATUS_INVALID_DEVICE_REQUEST);
         }
@@ -84,6 +86,18 @@ AFSFlushBuffers( IN PDEVICE_OBJECT LibDeviceObject,
             //
             try_return( ntStatus = STATUS_INVALID_PARAMETER);
         }
+
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "AFSFlushBuffers Acquiring Fcb SectionObject lock %p SHARED %08lX\n",
+                      &pFcb->NPFcb->SectionObjectResource,
+                      PsGetCurrentThread()));
+
+        AFSAcquireShared( &pFcb->NPFcb->SectionObjectResource,
+                          TRUE);
+
+        bReleaseSectionObject = TRUE;
+
         //
         // The flush consists of two parts.  We firstly flush our
         // cache (if we have one), then we tell the service to write
@@ -97,7 +111,7 @@ AFSFlushBuffers( IN PDEVICE_OBJECT LibDeviceObject,
             if (!NT_SUCCESS( iosb.Status ))
             {
 
-                AFSDbgLogMsg( AFS_SUBSYSTEM_IO_PROCESSING,
+                AFSDbgTrace(( AFS_SUBSYSTEM_IO_PROCESSING,
                               AFS_TRACE_LEVEL_ERROR,
                               "AFSFlushBuffers CcFlushCache [1] failure FID %08lX-%08lX-%08lX-%08lX Status 0x%08lX Bytes 0x%08lX\n",
                               pFcb->ObjectInformation->FileId.Cell,
@@ -105,7 +119,7 @@ AFSFlushBuffers( IN PDEVICE_OBJECT LibDeviceObject,
                               pFcb->ObjectInformation->FileId.Vnode,
                               pFcb->ObjectInformation->FileId.Unique,
                               iosb.Status,
-                              iosb.Information);
+                              iosb.Information));
 
                 try_return( ntStatus = iosb.Status );
             }
@@ -115,6 +129,17 @@ AFSFlushBuffers( IN PDEVICE_OBJECT LibDeviceObject,
 
             try_return( ntStatus = GetExceptionCode());
         }
+
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "AFSFlushBuffers Releasing Fcb SectionObject lock %p SHARED %08lX\n",
+                      &pFcb->NPFcb->SectionObjectResource,
+                      PsGetCurrentThread()));
+
+        AFSReleaseResource( &pFcb->NPFcb->SectionObjectResource);
+
+        bReleaseSectionObject = FALSE;
+
         //
         // Now, flush to the server - if there is stuff to do
         //
@@ -126,13 +151,26 @@ AFSFlushBuffers( IN PDEVICE_OBJECT LibDeviceObject,
         {
 
             AFSReleaseExtentsWithFlush( pFcb,
-                                        &pCcb->AuthGroup);
+                                        &pCcb->AuthGroup,
+                                        TRUE);
 
             ntStatus = STATUS_SUCCESS;
         }
 
 try_exit:
 
+        if ( bReleaseSectionObject)
+        {
+
+            AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
+                          AFS_TRACE_LEVEL_VERBOSE,
+                          "AFSFlushBuffers Releasing Fcb SectionObject lock %p SHARED %08lX\n",
+                          &pFcb->NPFcb->SectionObjectResource,
+                          PsGetCurrentThread()));
+
+            AFSReleaseResource( &pFcb->NPFcb->SectionObjectResource);
+        }
+
         AFSCompleteRequest( Irp, ntStatus);
     }
 
