@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -18,7 +18,7 @@
 #ifndef _OSI_MACHDEP_H_
 #define _OSI_MACHDEP_H_
 
-#ifdef AFS_SUN57_64BIT_ENV
+#ifdef AFS_SUN5_64BIT_ENV
 #include <sys/model.h>		/* for get_udatamodel() */
 #endif
 
@@ -31,11 +31,12 @@
   */
 #define NFSSRV 		"/kernel/misc/nfssrv"
 #define NFSSRV_V9 	"/kernel/misc/sparcv9/nfssrv"
+#define NFSSRV_AMD64 	"/kernel/misc/amd64/nfssrv"
 
 typedef struct cred afs_ucred_t;
 typedef struct proc afs_proc_t;
 
-/* 
+/*
  * Time related macros
  */
 #define	afs_hz	    hz
@@ -53,10 +54,19 @@ local_osi_Time()
 #define osi_Time() (hrestime.tv_sec)
 #endif
 
-#undef afs_osi_Alloc_NoSleep
-extern void *afs_osi_Alloc_NoSleep(size_t size);
+#define osi_vnhold(avc, r)  do {    \
+    struct vnode *vp = AFSTOV(avc); \
+    uint_t prevcount;               \
+                                    \
+    mutex_enter(&vp->v_lock);       \
+    prevcount = vp->v_count++;      \
+    mutex_exit(&vp->v_lock);        \
+                                    \
+    if (prevcount == 0) {           \
+	VFS_HOLD(afs_globalVFS);    \
+    }                               \
+} while(0)
 
-#define osi_vnhold(avc, r)  do { VN_HOLD(AFSTOV(avc)); } while(0)
 #define gop_rdwr(rw,gp,base,len,offset,segflg,ioflag,ulimit,cr,aresid) \
   vn_rdwr((rw),(gp),(base),(len),(offset),(segflg),(ioflag),(ulimit),(cr),(aresid))
 #define gop_lookupname(fnamep,segflg,followlink,compvpp) \
@@ -72,7 +82,7 @@ extern void *afs_osi_Alloc_NoSleep(size_t size);
 #endif
 
 /*
- * Global lock support. 
+ * Global lock support.
  */
 #include <sys/mutex.h>
 extern kmutex_t afs_global_lock;
@@ -99,15 +109,13 @@ extern kmutex_t afs_global_lock;
 #define	IO_SYNC		FSYNC
 #endif
 
-#if	defined(AFS_SUN56_ENV)
 /*
-** Macro returns 1 if file is larger than 2GB; else returns 0 
+** Macro returns 1 if file is larger than 2GB; else returns 0
 */
 #undef AfsLargeFileUio
 #define AfsLargeFileUio(uio)       ( (uio)->_uio_offset._p._u ? 1 : 0 )
 #undef AfsLargeFileSize
 #define AfsLargeFileSize(pos, off) ( ((offset_t)(pos)+(offset_t)(off) > (offset_t)0x7fffffff)?1:0)
-#endif
 
 #if defined(AFS_SUN510_ENV)
 #include "h/sunddi.h"
@@ -127,4 +135,7 @@ struct afs_ifinfo {
   int         metric;
   ipaddr_t    dstaddr;
 };
+
+#define osi_procname(procname, size) strncpy(procname, PTOU(ttoproc(curthread))->u_comm, size)
+
 #endif /* _OSI_MACHDEP_H_ */
