@@ -222,7 +222,7 @@ openOutFile(struct volumeHeader *headerPtr)
     if (ask) {
 	first = 1;
 	skip = 0;
-	printf("Press return to retreive volume %s (%u) to file %s; " "s"
+	printf("Press return to retrieve volume %s (%u) to file %s; " "s"
 	       " to skip\n", headerPtr->volumeName, headerPtr->volumeID,
 	       filename);
 	do {
@@ -421,7 +421,7 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
     /*
      * Initialize the tape block buffers
      */
-    tapeblock1 = malloc(3 * 16384);
+    tapeblock1 = (char *)malloc(3 * 16384);
     if (tapeblock1 == NULL) {
 	printf("Failed to allocate I/O buffers.\n");
 	exit(1);
