@@ -1,5 +1,5 @@
 /*
- * Thanks to Jim Rees and University of Michigan CITI, for the initial 
+ * Thanks to Jim Rees and University of Michigan CITI, for the initial
  * OpenBSD porting work.
  */
 
@@ -13,7 +13,6 @@
 #define AFS_XBSD_ENV		1	/* {Free,Open,Net}BSD */
 
 #define AFS_NAMEI_ENV		1	/* User space interface to file system */
-#define AFS_64BIT_ENV		1
 #define AFS_64BIT_CLIENT	1
 #define AFS_64BIT_IOPS_ENV	1	/* Needed for NAMEI */
 
@@ -50,6 +49,10 @@
 #include <afs/afs_sysnames.h>
 #endif
 
+#ifndef TAILQ_ENTRY
+#include <sys/queue.h>
+#endif
+
 /* Extra kernel definitions (from kdefs file) */
 #ifdef _KERNEL
 #ifdef MULTIPROCESSOR
