@@ -69,25 +69,6 @@ KA_rxstat_userok(struct rx_call *call)
     return afsconf_SuperUser(KA_conf, call, NULL);
 }
 
-/**
- * Return true if this name is a member of the local realm.
- */
-static int
-KA_IsLocalRealmMatch(void *rock, char *name, char *inst, char *cell)
-{
-    struct afsconf_dir *dir = (struct afsconf_dir *)rock;
-    afs_int32 islocal = 0;	/* default to no */
-    int code;
-
-    code = afsconf_IsLocalRealmMatch(dir, &islocal, name, inst, cell);
-    if (code) {
-	ViceLog(0,
-		("Failed local realm check; code=%d, name=%s, inst=%s, cell=%s\n",
-		 code, name, inst, cell));
-    }
-    return islocal;
-}
-
 afs_int32
 es_Report(char *fmt, ...)
 {
@@ -226,7 +207,7 @@ main(int argc, char *argv[])
     cellservdb = AFSDIR_SERVER_ETC_DIRPATH;
     dbpath = AFSDIR_SERVER_KADB_FILEPATH;
     strcompose(default_lclpath, AFSDIR_PATH_MAX, AFSDIR_SERVER_LOCAL_DIRPATH,
-	       "/", AFSDIR_KADB_FILE, (char *)NULL);
+	       "/", AFSDIR_KADB_FILE, NULL);
     lclpath = default_lclpath;
 
     debugOutput = 0;
@@ -350,9 +331,6 @@ main(int argc, char *argv[])
 	ViceLog(0, ("Using server list from %s cell database.\n", cell));
     }
 
-    /* initialize audit user check */
-    osi_audit_set_user_check(KA_conf, KA_IsLocalRealmMatch);
-
     /* initialize ubik */
     if (level == rxkad_clear)
 	ubik_SetClientSecurityProcs(afsconf_ClientAuth, afsconf_UpToDate,
@@ -393,9 +371,6 @@ main(int argc, char *argv[])
 	}
     }
 
-    /* Disable jumbograms */
-    rx_SetNoJumbo();
-
     if (servers)
 	code =
 	    ubik_ServerInit(myHost, htons(AFSCONF_KAUTHPORT), serverList,
@@ -412,6 +387,9 @@ main(int argc, char *argv[])
 
     sca[RX_SCINDEX_NULL] = rxnull_NewServerSecurityObject();
 
+    /* Disable jumbograms */
+    rx_SetNoJumbo();
+
     tservice =
 	rx_NewServiceHost(host, 0, KA_AUTHENTICATION_SERVICE,
 			  "AuthenticationService", sca, 1, KAA_ExecuteRequest);
