@@ -68,6 +68,7 @@ int combinepackages = 0;
 int PackageIndex = -1;
 int PerProcCounter = 0;
 int Multi_Init = 0;
+int StatIndex = -1;
 
 /*
  * Character arrays to keep list of function names as we process the file
@@ -142,9 +143,12 @@ static void er_ProcDeclExterns_setup(void);
 static void er_ProcProcsArray_setup(void);
 static void er_ProcMainBody_setup(void);
 static void er_HeadofOldStyleProc_setup(void);
+static void er_HeadofOldStyleProc_setup2(void);
 static void er_BodyofOldStyleProc_setup(void);
+static void er_BodyofOldStyleProc_setup2(void);
 static void proc_er_case(definition * defp);
 static void er_TailofOldStyleProc_setup(void);
+static void er_TailofOldStyleProc_setup2(void);
 
 
 
@@ -622,6 +626,7 @@ def_statindex(definition * defp)
     defp->def_name = name;
     defp->def.co = tok.str;
     PackageStatIndex[PackageIndex] = name;
+    StatIndex = atoi(tok.str);
 }
 
 static void
@@ -840,6 +845,7 @@ check_proc(definition * defp, token * tokp, int noname)
 	    prefix, PackagePrefix[PackageIndex], defp->pc.proc_name);
 
     function_list_index++;
+    defp->statindex = no_of_stat_funcs;
     no_of_stat_funcs_header[PackageIndex]++;
     no_of_stat_funcs++;
     *Proc_listp = NULL;
@@ -929,7 +935,7 @@ generate_code(definition * defp, int proc_split_flag, int multi_flag)
 	if (Sflag || cflag)
 	    ss_Proc_CodeGeneration(defp);
     }
-    if (Sflag || (cflag && xflag && !proc_split_flag))
+    if (Sflag || (cflag && xflag && !proc_split_flag) || hflag)
 	STOREVAL(&proc_defined[PackageIndex], defp);
 }
 
@@ -1147,16 +1153,21 @@ cs_ProcName_setup(definition * defp, char *procheader, int split_flag)
     if (!cflag) {
 	for (plist = defp->pc.plists; plist; plist = plist->next) {
 	    if (plist->component_kind == DEF_PARAM) {
+		f_print(fout, ",");
 		if (ansic_flag) {
+		    if (plist->pl.param_kind == DEF_INPARAM &&
+			strcmp(plist->pl.param_type, "char *") == 0) {
+			f_print(fout, "const ");
+		    }
 		    if (plist->pl.param_flag & OUT_STRING) {
-			f_print(fout, ",%s *%s", plist->pl.param_type,
+			f_print(fout, "%s *%s", plist->pl.param_type,
 				plist->pl.param_name);
 		    } else {
-			f_print(fout, ",%s %s", plist->pl.param_type,
+			f_print(fout, "%s %s", plist->pl.param_type,
 				plist->pl.param_name);
 		    }
 		} else {
-		    f_print(fout, ", %s", plist->pl.param_name);
+		    f_print(fout, " %s", plist->pl.param_name);
 		    plist->pl.param_flag &= ~PROCESSED_PARAM;
 		}
 	    }
@@ -1735,17 +1746,22 @@ ucs_ProcName_setup(definition * defp, char *procheader, int split_flag)
     if (!cflag) {
 	for (plist = defp->pc.plists; plist; plist = plist->next) {
 	    if (plist->component_kind == DEF_PARAM) {
+		f_print(fout, ",");
 		if (ansic_flag) {
+		    if (plist->pl.param_kind == DEF_INPARAM &&
+			strcmp(plist->pl.param_type, "char *") == 0) {
+			f_print(fout, "const ");
+		    }
 		    if (plist->pl.param_flag & OUT_STRING) {
-			f_print(fout, ",%s *%s", plist->pl.param_type,
+			f_print(fout, "%s *%s", plist->pl.param_type,
 				plist->pl.param_name);
 		    } else {
-			f_print(fout, ",%s %s", plist->pl.param_type,
+			f_print(fout, "%s %s", plist->pl.param_type,
 				plist->pl.param_name);
 		    }
 		} else {
 		    plist->pl.param_flag &= ~PROCESSED_PARAM;
-		    f_print(fout, ", %s", plist->pl.param_name);
+		    f_print(fout, " %s", plist->pl.param_name);
 		}
 	    }
 	}
@@ -1960,6 +1976,9 @@ er_Proc_CodeGeneration(void)
 	    er_HeadofOldStyleProc_setup();
 	    er_BodyofOldStyleProc_setup();
 	    er_TailofOldStyleProc_setup();
+	    er_HeadofOldStyleProc_setup2();
+	    er_BodyofOldStyleProc_setup2();
+	    er_TailofOldStyleProc_setup2();
 	} else {
 	    er_ProcDeclExterns_setup();
 	    er_ProcProcsArray_setup();
@@ -2044,6 +2063,13 @@ er_ProcMainBody_setup(void)
 		PackagePrefix[PackageIndex], PackagePrefix[PackageIndex]);
 	f_print(fout, "\treturn opnames%d[op - %sLOWEST_OPCODE];\n}\n",
 		PackageIndex, PackagePrefix[PackageIndex]);
+	f_print(fout, "struct %sstats *%sOpCodeStats(int op)\n{\n",
+		PackagePrefix[PackageIndex], PackagePrefix[PackageIndex]);
+	f_print(fout, "\tif (op < %sLOWEST_OPCODE || op > %sHIGHEST_OPCODE)\n\t\treturn NULL;\n",
+		PackagePrefix[PackageIndex], PackagePrefix[PackageIndex]);
+	f_print(fout, "\treturn NULL;/*%d %s*/\n}\n",
+		PackageIndex, PackagePrefix[PackageIndex]);
+
 	return;
     }
     f_print(fout, "int %s%sExecuteRequest(struct rx_call *z_call)\n",
@@ -2063,6 +2089,14 @@ er_ProcMainBody_setup(void)
     f_print(fout, "\treturn hton_syserr_conv(z_result);\n}\n");
 }
 
+static void
+er_HeadofOldStyleProc_setup2(void)
+{
+    if ( cflag ) {
+	f_print(fout, "int %sOpCodeIndex(int op)\n{\n", (combinepackages ? MasterPrefix : PackagePrefix[PackageIndex]));
+        f_print(fout, "\tswitch (op) {\n");
+    }
+}
 
 static void
 er_HeadofOldStyleProc_setup(void)
@@ -2132,6 +2166,44 @@ proc_er_case(definition * defp)
     f_print(fout, "\t\t\tbreak;\n");
 }
 
+static void
+proc_op_case(definition * defp)
+{
+    f_print(fout, "\t\tcase %d:", defp->pc.proc_opcodenum);
+    f_print(fout, "\treturn %d;\n",
+	    defp->statindex);
+}
+
+static void
+er_BodyofOldStyleProc_setup2(void)
+{
+    list *listp;
+
+    if (!cflag)
+	return;
+    if (combinepackages) {
+	int temp = PackageIndex;
+	for (PackageIndex = 0; PackageIndex <= temp; PackageIndex++) {
+	    for (listp = proc_defined[PackageIndex]; listp != NULL;
+		 listp = listp->next)
+		proc_op_case((definition *) listp->val);
+	}
+	PackageIndex = temp;
+    } else {
+	for (listp = proc_defined[PackageIndex]; listp != NULL;
+	     listp = listp->next)
+	    proc_op_case((definition *) listp->val);
+    }
+}
+
+static void
+er_TailofOldStyleProc_setup2(void)
+{
+    if ( cflag ) {
+	f_print(fout, "\t\tdefault:\n");
+	f_print(fout, "\t\t\treturn -1;\n\t}\n}\n");
+    }
+}
 
 static void
 er_TailofOldStyleProc_setup(void)
@@ -2157,9 +2229,12 @@ h_ProcMainBody_setup(void)
 static void
 h_HeadofOldStyleProc_setup(void)
 {
+    char *pprefix = (combinepackages ? MasterPrefix :
+		     PackagePrefix[PackageIndex]);
+    f_print(fout,"\nstruct %sstats{\n\tint statsver;\n};", pprefix);
     f_print(fout,"\nextern int %s%sExecuteRequest(struct rx_call *);\n",
-	    prefix,
-	    (combinepackages ? MasterPrefix : PackagePrefix[PackageIndex]));
+	    prefix, pprefix);
+    f_print(fout,"\nextern int %sOpCodeIndex(int op);\n", PackagePrefix[PackageIndex]);
 }
 
 void
@@ -2180,45 +2255,52 @@ h_Proc_CodeGeneration(void)
     PackageIndex = temp;
 }
 
+static void
+proc_h_case(definition * defp)
+{
+    f_print(fout, "#define opcode_%s%s \t((afs_uint64)((%uLL << 32) + %sOpCodeIndex(%u)))\n",
+	    defp->pc.proc_prefix, defp->pc.proc_name, StatIndex,
+	    defp->pc.proc_prefix, defp->pc.proc_opcodenum);
+}
+
+void
+h_opcode_stats_pkg(char *pprefix, int lowest, int highest, int nops,
+		  int statfuncs, char *ptype, list *proclist)
+{
+    list *listp;
+
+    if (!pprefix)
+	return;
+
+    f_print(fout,
+            "\n/* Opcode-related useful stats for %spackage: %s */\n",
+            ptype, pprefix);
+    f_print(fout, "#define %sLOWEST_OPCODE   %d\n", pprefix, lowest);
+    f_print(fout, "#define %sHIGHEST_OPCODE     %d\n", pprefix, highest);
+    f_print(fout, "#define %sNUMBER_OPCODES     %d\n\n", pprefix, nops);
+
+    for (listp = proclist; listp != NULL;
+         listp = listp->next)
+        proc_h_case((definition *) listp->val);
+
+    if (xflag) {
+        f_print(fout, "#define %sNO_OF_STAT_FUNCS\t%d\n\n",
+                pprefix, statfuncs);
+        f_print(fout, "AFS_RXGEN_EXPORT\n");
+        f_print(fout, "extern const char *%sfunction_names[];\n\n",
+                pprefix);
+    }
+}
+
 void
 h_opcode_stats(void)
 {
     if (combinepackages) {
-	f_print(fout,
-		"\n/* Opcode-related useful stats for Master package: %s */\n",
-		MasterPrefix);
-	f_print(fout, "#define %sLOWEST_OPCODE   %d\n", MasterPrefix,
-		master_lowest_opcode);
-	f_print(fout, "#define %sHIGHEST_OPCODE	%d\n", MasterPrefix,
-		master_highest_opcode);
-	f_print(fout, "#define %sNUMBER_OPCODES	%d\n\n", MasterPrefix,
-		master_no_of_opcodes);
-	if (xflag) {
-	    f_print(fout, "#define %sNO_OF_STAT_FUNCS\t%d\n\n", MasterPrefix,
-		    no_of_stat_funcs_header[0]);
-	    f_print(fout, "AFS_RXGEN_EXPORT\n");
-	    f_print(fout, "extern const char *%sfunction_names[];\n\n",
-		    MasterPrefix);
-	}
+	h_opcode_stats_pkg(MasterPrefix, master_lowest_opcode, master_highest_opcode, master_no_of_opcodes, no_of_stat_funcs_header[0], "Master ", proc_defined[0]);
     } else {
 	int i;
 	for (i = 0; i <= PackageIndex; i++) {
-	    f_print(fout,
-		    "\n/* Opcode-related useful stats for package: %s */\n",
-		    PackagePrefix[i]);
-	    f_print(fout, "#define %sLOWEST_OPCODE   %d\n", PackagePrefix[i],
-		    lowest_opcode[i]);
-	    f_print(fout, "#define %sHIGHEST_OPCODE	%d\n",
-		    PackagePrefix[i], highest_opcode[i]);
-	    f_print(fout, "#define %sNUMBER_OPCODES	%d\n\n",
-		    PackagePrefix[i], no_of_opcodes[i]);
-	    if (xflag) {
-		f_print(fout, "#define %sNO_OF_STAT_FUNCS\t%d\n\n",
-			PackagePrefix[i], no_of_stat_funcs_header[i]);
-		f_print(fout, "AFS_RXGEN_EXPORT\n");
-		f_print(fout, "extern const char *%sfunction_names[];\n\n",
-			PackagePrefix[i]);
-	    }
+	    h_opcode_stats_pkg(PackagePrefix[i], lowest_opcode[i], highest_opcode[i], no_of_opcodes[i], no_of_stat_funcs_header[i], "", proc_defined[i]);
 	}
     }
 }
