@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -78,6 +78,7 @@ struct rxkad_sprivate {
     int (*get_key) (void *, int,
 		    struct ktc_encryptionKey *);
 				/* func. of kvno and server key ptr */
+    rxkad_get_key_enctype_func get_key_enctype;
     int (*user_ok) (char *, char *,
 		    char *, afs_int32);
 				/* func called with new client name */
